package com.sun.org.apache.bcel.internal.classfile;

import java.util.Map;
import java.util.LinkedHashMap;
import java.util.HashMap;
import java.io.DataOutputStream;
import java.io.IOException;
import java.io.DataInput;

public final class ConstantUtf8 extends Constant
{
    private final String bytes;
    private static volatile int considered;
    private static volatile int hits;
    private static volatile int skipped;
    private static volatile int created;
    private static final int MAX_CACHED_SIZE = 200;
    private static final boolean BCEL_STATISTICS = false;
    
    static void printStats() {
        System.err.println("Cache hit " + ConstantUtf8.hits + "/" + ConstantUtf8.considered + ", " + ConstantUtf8.skipped + " skipped");
        System.err.println("Total of " + ConstantUtf8.created + " ConstantUtf8 objects created");
    }
    
    static void clearStats() {
        ConstantUtf8.hits = (ConstantUtf8.considered = (ConstantUtf8.skipped = (ConstantUtf8.created = 0)));
    }
    
    public static ConstantUtf8 getCachedInstance(String s) {
        if (s.length() > 200) {
            ++ConstantUtf8.skipped;
            return new ConstantUtf8(s);
        }
        ++ConstantUtf8.considered;
        synchronized (ConstantUtf8.class) {
            ConstantUtf8 result = (ConstantUtf8)ConstantUtf8.CACHE_HOLDER.CACHE.get(s);
            if (result != null) {
                ++ConstantUtf8.hits;
                return result;
            }
            result = new ConstantUtf8(s);
            ConstantUtf8.CACHE_HOLDER.CACHE.put(s, result);
            return result;
        }
    }
    
    public static ConstantUtf8 getInstance(String s) {
        return new ConstantUtf8(s);
    }
    
    public static ConstantUtf8 getInstance(DataInput input) throws IOException {
        return getInstance(input.readUTF());
    }
    
    public ConstantUtf8(ConstantUtf8 c) {
        this(c.getBytes());
    }
    
    ConstantUtf8(DataInput file) throws IOException {
        super((byte)1);
        this.bytes = file.readUTF();
        ++ConstantUtf8.created;
    }
    
    public ConstantUtf8(String bytes) {
        super((byte)1);
        if (bytes == null) {
            throw new IllegalArgumentException("bytes must not be null!");
        }
        this.bytes = bytes;
        ++ConstantUtf8.created;
        return;
    }
    
    @Override
    public void accept(Visitor v) {
        v.visitConstantUtf8(this);
    }
    
    @Override
    public final void dump(DataOutputStream file) throws IOException {
        file.writeByte((int)super.getTag());
        file.writeUTF(this.bytes);
    }
    
    public final String getBytes() {
        return this.bytes;
    }
    
    @Deprecated
    public final void setBytes(String bytes) {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public final String toString() {
        return super.toString() + "(\"" + Utility.replace(this.bytes, "\n", "\\n") + "\")";
    }
    
    static {
        ConstantUtf8.considered = 0;
        ConstantUtf8.hits = 0;
        ConstantUtf8.skipped = 0;
        ConstantUtf8.created = 0;
    }
    
    private static class CACHE_HOLDER
    {
        private static final int MAX_CACHE_ENTRIES = 20000;
        private static final int INITIAL_CACHE_CAPACITY = 26666;
        private static final HashMap<String, ConstantUtf8> CACHE;
        
        static {
            CACHE = new LinkedHashMap<String, ConstantUtf8>(26666, 0.75f, true) {
                private static final long serialVersionUID = -8506975356158971766L;
                
                @Override
                protected boolean removeEldestEntry(Map.Entry<String, ConstantUtf8> eldest) {
                    return this.size() > 20000;
                }
            };
        }
    }
}
